package de.ejb.remote.server.security;

import java.io.Serializable;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.Principal;

public class MyAppPrincipal implements Principal, Serializable
{
	private static final long serialVersionUID = -4384014180512184923L;

	private final String name;

	private final String applicationName;

	private final long clientSessionID;

	private final String osUsername;

	private final String hostname;


	public MyAppPrincipal(final String name)
	{
		this(name, "");
	}


	public MyAppPrincipal(final String name, final String appName)
	{
		if(name == null)
		{
			throw new IllegalArgumentException("null name is illegal");
		}
		this.name = name;
		this.applicationName = appName;
		this.clientSessionID = System.nanoTime();
		this.osUsername = System.getProperty("user.name");
		this.hostname = getHostName();
	}


	@Override
	public String getName()
	{
		return name;
	}


	/**
	 * gets the name of the application of this session
	 *
	 * @return the application name
	 */
	public String getApplicationName()
	{
		return applicationName;
	}


	/**
	 * returns an client session identifier
	 *
	 * @return the client session id
	 */
	public long getClientSessionID()
	{
		return clientSessionID;
	}


	/**
	 * gets the clients os username
	 *
	 * @return the os username
	 */
	public String getOsUsername()
	{
		return osUsername;
	}


	/**
	 * gets the clients computers hostname
	 *
	 * @return the hostname
	 */
	public String getHostname()
	{
		return hostname;
	}


	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + ((applicationName == null) ? 0 : applicationName.hashCode());
		result = prime * result + (int) (clientSessionID ^ (clientSessionID >>> 32));
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}


	@Override
	public boolean equals(final Object obj)
	{
		if(this == obj)
		{
			return true;
		}
		if(obj == null)
		{
			return false;
		}
		if(getClass() != obj.getClass())
		{
			return false;
		}
		final MyAppPrincipal other = (MyAppPrincipal) obj;
		if(applicationName == null)
		{
			if(other.applicationName != null)
			{
				return false;
			}
		}
		else if(!applicationName.equals(other.applicationName))
		{
			return false;
		}
		if(clientSessionID != other.clientSessionID)
		{
			return false;
		}
		if(name == null)
		{
			if(other.name != null)
			{
				return false;
			}
		}
		else if(!name.equals(other.name))
		{
			return false;
		}
		return true;
	}


	/**
	 * Gets the host name.
	 *
	 * @return the host name
	 */
	private String getHostName()
	{
		String result = null;

		try
		{
			final InetAddress addr = InetAddress.getLocalHost();
			if(addr != null)
			{
				result = addr.getHostName();
			}
		}
		catch(final UnknownHostException ex)
		{
			result = null;
		}
		return result;
	}


	@Override
	public String toString()
	{
		return "MyAppPrincipal [name=" + name + ", applicationName=" + applicationName + ", clientSessionID=" + clientSessionID + ", osUsername=" + osUsername + ", hostname=" + hostname + "]";
	}

}
